/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifndef PSKCODEC_H
#define PSKCODEC_H		1

#include <math.h>
#include "common.h"

/* Signal sliding average window size */
#define SIGNAL_AVE_WINDOW	2

/* The offset between the start bit
 * detection and first RTTY data bit */
#define STARTBIT_CORRECTION		1

/* Scale factor for syncro display */
#define SYNCHRO_SCALE	8

/* Scale factor for scope display */
#define SCOPE_SCALE		2

/* Buffer size of RTTY decoder buffers. We need to
 * buffer 3 RTTY characters of total 32 signal samples
 * (1 start, 5 data, 2 stop bits) of 4 samples each */
#define DECODER_BUFF_SIZE	32 * NUM_BUFFERED_CHARS

/* Definitions for RTTY to ASCII conversion */
#define NUM_ITA2_CHARS		64
#define NUM_BAUDOT_CHARS	32
#define FIGS_SHIFT_IDX		27
#define LTRS_SHIFT_IDX		31

/* The characters in this table can be changed before compilation
 * if a different character code is needed. PLEASE NOTE that the
 * non-printable characters '|', '>', '<', '^', 'v', 'e', 'b' must
 * not be changed, otherwise xritty will not work properly */
#define ITA2_CODE \
{ \
  '|','E','>','A',' ','S','I','U','<','D','R','J','N','F','C','K',	\
  'T','Z','L','W','H','Y','P','Q','O','B','G','^','M','X','V','v',	\
  '|','3','>','-',' ','\'','8','7','<','e','4','b',',','!',':','(', \
  '5','+',')','2','#','6','0','1','9','?','&','^','.','/',';','v'	\
}

enum
{
  MARK = 0,
  SPACE,
  TONE
};

/* Definitions for ASCII to Morse code conversion */
/* Number of chars in Morse table */
#define NUM_MORSE_CHARS     55

/* ASCII equivalents to Morse hex code. Last */
/* one (*) used for unrecognized characters. */
#define ASCII_CHAR \
{ \
  'A','B','C','D','E','F','G','H','I','J','K','L','M',  \
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',  \
  '1','2','3','4','5','6','7','8','9','0','.',',',':',  \
  '?','\'','-','/','(','"',';','$','#','<','!','@',']', \
  '=','~',' '										    \
}

/* Hex equivalents to Morse code chars above. Formed   */
/* by starting with a 1 and following with a 0 for     */
/* dash and 1 for dit e.g: A = ditdah = 101 = Hex 0x06 */
#define MORSE_CODE \
{ \
  0x06,0x17,0x15,0x0b,0x03,0x1d,0x09,0x1f,0x07,0x18,0x0a,0x1b,0x04, \
  0x05,0x08,0x19,0x12,0x0d,0x0f,0x02,0x0e,0x1e,0x0c,0x16,0x14,0x13, \
  0x30,0x38,0x3c,0x3e,0x3f,0x2f,0x27,0x23,0x21,0x20,0x6a,0x4c,0x47, \
  0x73,0x61,0x5e,0x2d,0x52,0x6d,0x55,0xf6,0x35,0x7a,0x2a,0x37,0x29, \
  0x2e,0xff,0x01 												    \
}

#endif

