/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "operation.h"
#include "shared.h"

/*------------------------------------------------------------------------*/

/*  Transmit_Keybd()
 *
 *  Transmits directly from the keyboard
 */

  gboolean
Transmit_Keybd( gpointer data )
{
  /* Text buffer marker */
  static GtkTextIter iter;

  /* First call of function flag */
  static gboolean first_call = TRUE;

  /* Pointer to current char in keybd buffer */
  static int key_pnt = 0;

  /*** Cancel Transmit if Tx flag cleared ***/
  if( isFlagClear(TRANSMIT_KEYBD) )
  {
	/* Switch to CW and PTT OFF */
	if( isFlagClear(TRANSMIT_MACRO) )
	{
	  Transmit_Postamble();
	  Close_Playback();
	  ClearFlag( TRANSMIT_MODE );
	}

	first_call = TRUE;
	return( FALSE );
  } /* if( isFlagClear(TRANSMIT_KEYBD) ) */

  /*** Initialize if needed ***/
  if( first_call )
  {
	gtk_widget_grab_focus(
		Builder_Get_Object( main_window_builder, "tx_textview") );

	if( isFlagClear(TRANSMIT_MODE) )
	  if( !Initialize_Transmit(data) )
	  {
		first_call = TRUE;
		return( FALSE );
	  }

	first_call = FALSE;
	return( TRUE );
  } /* if( first_call ) */

  /*** Process keystrokes from user ***/
  if( key_cnt != key_pnt )
  {
	/* Set up TRANSMIT text view */
	gtk_text_buffer_get_iter_at_offset(
		tx_text_buffer, &iter,
		gtk_text_buffer_get_char_count
		(tx_text_buffer) );

	/* Capitalize letters if needed */
	if( (key_buff[key_pnt] > 0x60) &&
		(key_buff[key_pnt] < 0x7b) )
	  key_buff[key_pnt] -= 0x20;

	/* Send LF and CR on Return */
	if( key_buff[key_pnt] == GDK_KEY_Return )
	{
	  char_cnt = 0; /* Reset word wrap */
	  if( !Transmit_RTTY_Character(CR) ||
		  !Transmit_RTTY_Character(LF) )
	  {
		Close_Playback();
		return( FALSE );
	  }

	  Print_Character
		( GDK_KEY_Return, tx_text_buffer,
		  tx_scrolledwindow, &iter );

	  /* Reset key buff pointer */
	  if( ++key_pnt >= KEY_BUFF_SIZE )
		key_pnt = 0;
	  return( TRUE );
	}

	/* Do word wrapping */
	if( (char_cnt >= rc_data.word_wrap) &&
		(key_buff[key_pnt] == ' ') )
	{
	  char_cnt = 0; /* Reset word wrap */
	  if( !Transmit_RTTY_Character(CR)||
		  !Transmit_RTTY_Character(LF) )
	  {
		Close_Playback();
		return( FALSE );
	  }

	  Print_Character(
		  GDK_KEY_Return, tx_text_buffer,
		  tx_scrolledwindow, &iter );

	  /* Reset key buff pointer */
	  if( ++key_pnt >= KEY_BUFF_SIZE )
		key_pnt = 0;
	  return( TRUE );
	}

	/* Backspace cursor */
	if( key_buff[key_pnt] == GDK_KEY_BackSpace )
	{
	  if( char_cnt ) char_cnt--;
	  key_buff[key_pnt] = GDK_KEY_BackSpace;
	}

	/* Transmit keyboard char and record */
	char_cnt++;
	if( !Transmit_RTTY_Character((int)key_buff[key_pnt]) )
	{
	  Close_Playback();
	  return( FALSE );
	}

	Print_Character(
		(int)key_buff[key_pnt],
		tx_text_buffer,
		tx_scrolledwindow,
		&iter );

	/* Reset key buff pointer */
	if( ++key_pnt >= KEY_BUFF_SIZE )
	  key_pnt = 0;
  }
  else
  {
	/* Transmit LTRS shift character in idle periods */
	if( !Transmit_RTTY_Character('v') )
	{
	  Close_Playback();
	  return( FALSE );
	}
  }

  return( TRUE );
} /* End of Transmit_Keybd() */

/*------------------------------------------------------------------------*/

/*  Transmit_Macro()
 *
 *  Transmits a pre-recorded Macro
 */

  gboolean
Transmit_Macro( gpointer data )
{
  /* Text buffer marker */
  static GtkTextIter iter;

  static int
	macro_cnt = 0, /* Count macro characters tx'd */
    tag_idx   = 0; /* Index for transmitting tags */

  /* First call of function flag */
  static gboolean first_call = TRUE;

  /* Tags for entering fields in macros */
  char *tags[NUM_OF_TAGS] = TAG_TABLE;

  /* String within a tag <> */
  char tag_str[13];

  /* Tag replacement string */
  static char tag_rep[22];


  /* Abort macro if user disabled, remain in Tx mode */
  /* Stop macro on '~' character, remain in Tx mode  */
  if( isFlagClear(TRANSMIT_MACRO) ||
	  (macro[macro_idx][macro_cnt] == '~') )
  {
	int ok = Transmit_RTTY_Character( NULL_CHAR );

	fflush( qso_record_fp );
	macro_cnt = tag_idx = 0;
	first_call = TRUE;
	ClearFlag( TRANSMIT_MACRO |	TRANSMIT_TAG );

	if( ok )
	{
	  SetFlag( TRANSMIT_KEYBD );
	  g_idle_add( Transmit_Keybd, "from_macro" );
	}
	return( FALSE );
  }

  /* Stop macro at string terminator, */
  /* or CW id char (*), go to Rx mode */
  if( (macro[macro_idx][macro_cnt] == '\0') ||
	  (macro[macro_idx][macro_cnt] == '*') )
  {
	/* Flush qso record file */
	fflush( qso_record_fp );

	/* Transmit postamble */
	if( !Transmit_Postamble() )
	{
	  Close_Playback();
	  fflush( qso_record_fp );
	  return( FALSE );
	}

	/* Transmit Morse ID if '*' */
	if( macro[macro_idx][macro_cnt] == '*' )
	{
	  char mesg[MESG_SIZE];
	  snprintf( mesg, sizeof(mesg),
		  "  DE %s  TU", rc_data.call );
	  Morse_Transmit( mesg );
	}

	macro_cnt = tag_idx = 0;
	first_call = TRUE;
	ClearFlag(
		TRANSMIT_MACRO |
		TRANSMIT_KEYBD |
		TRANSMIT_MODE  |
		TRANSMIT_TAG );

	Close_Playback();
	SetFlag( RECEIVE_MODE );
	g_idle_add( Receive_Mode, NULL );
	return( FALSE );
  }

  /*** Initialize if needed ***/
  if( first_call )
  {
	first_call = FALSE;
	if( isFlagClear(TRANSMIT_MODE) )
	  if( !Initialize_Transmit(data) )
	  {
		first_call = TRUE;
		return( FALSE );
	  }

	return( TRUE );
  } /* if( first_call ) */

  /* Set up TRANSMIT text view */
  gtk_text_buffer_get_iter_at_offset(
	  tx_text_buffer, &iter,
	  gtk_text_buffer_get_char_count
	  (tx_text_buffer) );

  /* If Macro begins with a ^ transmit new line */
  if( macro[macro_idx][macro_cnt] == '^' )
  {
	if( !Transmit_RTTY_Character(CR) ||
		!Transmit_RTTY_Character(LF) )
	{
	  Close_Playback();
	  return( FALSE );
	}

	Print_Character( LF, tx_text_buffer,
		tx_scrolledwindow, &iter );
	macro_cnt++;
	return( TRUE );
  }

  /*** Look for tags and substitude accordingly ***/
  if( (macro[macro_idx][macro_cnt] == '<') &&
	  isFlagClear(TRANSMIT_TAG) )
  {
	int idx;

	/*** Copy tag string to buffer and identify ***/
	SetFlag( TRANSMIT_TAG );
	macro_cnt++; /* Move into tag field */

	/* Copy line to tag, max 12 chars or EOS */
	idx = 0;
	size_t siz = sizeof( tag_str ) - 1;
	while( (idx < (int)siz) &&
		   (macro[macro_idx][macro_cnt] != '>') )
	{
	  tag_str[idx] = macro[macro_idx][macro_cnt];
	  idx++;
	  macro_cnt++;
	}

	/* Terminate tag buffer and advance macro buffer */
	tag_str[idx] = '\0';
	macro_cnt++;

	/* Identify tag and substitude accordingly */
	idx = 0;
	while( idx < NUM_OF_TAGS )
	{
	  if( strcmp(tag_str, tags[idx]) == 0 )	break;
	  idx++;
	}

	/* Abort if tag unidentified */
	if( idx == NUM_OF_TAGS )
	{
	  Error_Dialog(
		  _("Error reading xrittyrc file\n"\
			"A tag in a macro is invalid\n"\
			"Quit xritty and correct"), QUIT);
	  return( FALSE );
	}

	/* Replace tags */
	siz = sizeof( tag_rep );
	switch( idx )
	{
	  case 0: /* own-call */
		Strlcpy( tag_rep, rc_data.call, siz );
		break;

	  case 1: /* own-name */
		Strlcpy( tag_rep, rc_data.name, siz );
		break;

	  case 2: /* own-qth */
		Strlcpy( tag_rep, rc_data.qth, siz );
		break;

	  case 3: /* own-loc */
		Strlcpy( tag_rep, rc_data.loc, siz );
		break;

	  case 4: /* own-rst */
		Strlcpy( tag_rep, qso_record.my_rst, siz );
		break;

	  case 5: /* rem-call */
		Strlcpy( tag_rep, qso_record.dx_call, siz );
		break;

	  case 6: /* rem-name */
		Strlcpy( tag_rep, qso_record.dx_name, siz );
		break;

	  case 7: /* rem-qth */
		Strlcpy( tag_rep, qso_record.dx_qth, siz );
		break;

	  case 8: /* rem-loc */
		Strlcpy( tag_rep, qso_record.dx_loc, siz );
		break;

	  case 9: /* rem-rst */
		Strlcpy( tag_rep, qso_record.dx_rst, siz );
		break;

	  case 10: /* date-time */
		Strlcpy( tag_rep, qso_record.date, siz );
		Strlcat ( tag_rep, " ", siz );
		Strlcat( tag_rep, qso_record.time, siz );
		break;

	  case 11: /* op-freq, strip trailing spaces */
		{
		  int i;

		  Strlcpy( tag_rep, qso_record.freq, siz );
		  i = (int)strlen( tag_rep ) - 1;
		  for( ; i >= 0; i-- )
			if( tag_rep[i] == ' ' ) tag_rep[i] = '\0';
		}
		break;

	  case 12: /* app-version */
		snprintf( tag_rep, siz, "%s", PACKAGE_STRING );

	} /* switch( idx ) */

  } /* if( (macro[macro_idx][macro_cnt] == '<') && ) */

  /*** Transmit tag replacement ***/
  if( isFlagSet(TRANSMIT_TAG) )
  {
	if( tag_rep[tag_idx] != '\0' )
	{
	  /* Capitalize letters */
	  if( (tag_rep[tag_idx] > 0x60)  &&
		  (tag_rep[tag_idx] < 0x7b) )
		tag_rep[tag_idx] -= 0x20;

	  if( !Transmit_RTTY_Character(tag_rep[tag_idx]) )
	  {
		Close_Playback();
		return( FALSE );
	  }

	  Print_Character(
		  tag_rep[tag_idx],
		  tx_text_buffer,
		  tx_scrolledwindow,
		  &iter );
	  tag_idx++;

	} /* if( (tag_rep[tag_idx] != '\0') ) */
	else
	{
	  ClearFlag( TRANSMIT_TAG );
	  tag_idx = 0;
	}

	return( TRUE );
  } /* if( isFlagSet(TRANSMIT_TAG) && */

  /* Capitalize letters if needed */
  if( (macro[macro_idx][macro_cnt] > 0x60) &&
	  (macro[macro_idx][macro_cnt] < 0x7b) )
	macro[macro_idx][macro_cnt] -= 0x20;

  /* Transmit a char from Macro */
  if( !Transmit_RTTY_Character(macro[macro_idx][macro_cnt]) )
  {
	Close_Playback();
	return( FALSE );
  }
  Print_Character( macro[macro_idx][macro_cnt],
	  tx_text_buffer, tx_scrolledwindow, &iter );
  macro_cnt++;

  return( TRUE );
} /* Transmit_Macro() */

/*------------------------------------------------------------------------*/

/*  Open_Record_File()
 *
 *  Opens a file for raw QSO recording
 */

  static gboolean
Open_Record_File(void)
{
  /* File path for QSO record  */
  char qso_record_fpath[64];


  /* Open file for QSO Record, abort on error */
  if( isFlagSet(RECORD_QSO) )
  {
	/* Create a file for recording QSO's */
	if( qso_record_fp == NULL )
	{
	  /* Make file path for record file */
	  snprintf( qso_record_fpath, sizeof(qso_record_fpath),
		  "%s/xritty/record.txt", getenv("HOME") );

	  qso_record_fp = fopen( qso_record_fpath, "a" );
	  if( qso_record_fp == NULL )
	  {
		perror( qso_record_fpath );
		Error_Dialog(
			_("Failed to open QSO Record file\n"\
			  "Quit xritty and correct"), QUIT );
		return( FALSE );
	  }

	} /* if( QSO_record_fp == NULL ) */

  } /* if( isFlagSet(RECORD_QSO) ) */

  return( TRUE );
} /* Open_Record_File(void) */

/*------------------------------------------------------------------------*/

  gboolean
Receive_Mode( gpointer data )
{
  /* Text buffer marker */
  static GtkTextIter iter;

  /* First call of function flag */
  static gboolean first_call = TRUE;

  /* A character decoded from RTTY signals */
  int dec_chr;

  /* Received char count */
  static int chr_cnt = 0;


  /* Cancel Receive if Rx flag cleared */
  if( isFlagClear(RECEIVE_MODE) )
  {
	Close_Capture();
	first_call = TRUE;
	return( FALSE );
  }

  /*** Initialize ***/
  if( first_call )
  {
	int error;
	char mesg[MESG_SIZE];
	first_call = FALSE;

	/* Setup sound card, abort on error */
	mesg[0] = '\0';
	if( !Open_Capture( mesg, &error ) )
	{
	  if( error )
	  {
		Strlcat( mesg, _("\nError: "), sizeof(mesg) );
		Strlcat( mesg, snd_strerror(error), sizeof(mesg) );
	  }
	  Close_Capture();

	  ClearFlag( RECEIVE_MODE );
	  Set_TxRx_Labels();
	  first_call = TRUE;
	  Error_Dialog( mesg, QUIT );
	  return( FALSE );
	} /* if( !Setup_Sound_Card( mesg, &error ) ) */

	/* Set transceiver mode if CAT enabled, abort on error */
	if( isFlagSet(ENABLE_CAT) )
	{
	  if( !Open_Tcvr_Serial() || !Set_Tcvr_Mode() )
	  {
		ClearFlag( RECEIVE_MODE );
		Set_TxRx_Labels();
		Close_Capture();
		first_call = TRUE;
		return( FALSE );
	  }
	}
	else if( isFlagSet(ENABLE_RTS_DTR) )
	{
	  /* Open RTS control, abort on error */
	  if( !Open_RTS_Serial() || !Set_Tcvr_Mode() )
	  {
		ClearFlag( RECEIVE_MODE );
		Set_TxRx_Labels();
		Close_Capture();
		first_call = TRUE;
		return( FALSE );
	  }
	}

	/* Abort if QSO record file fails to open */
	if( !Open_Record_File() )
	{
	  ClearFlag( RECEIVE_MODE );
	  Set_TxRx_Labels();
	  Close_Capture();
	  first_call = TRUE;
	  return( FALSE );
	}

	/* Print new line to Rx viewer */
	gtk_text_buffer_get_iter_at_offset(
		rx_text_buffer, &iter,
		gtk_text_buffer_get_char_count(
		  rx_text_buffer) );

	Print_Character(
		LF, rx_text_buffer,
		rx_scrolledwindow, &iter );

	/* Reset char count on line feed */
	chr_cnt = 0;

	Set_TxRx_Labels();
	return( TRUE );
  } /* if( first_call ) */

  /*** Print characters to Receive window & file ***/
  /* Set up RECEIVE text view */
  gtk_text_buffer_get_iter_at_offset(
	  rx_text_buffer, &iter,
	  gtk_text_buffer_get_char_count
	  (rx_text_buffer) );

  /* Read a character, abort on dsp error */
  if( !Decode_RTTY_Character(&dec_chr) )
  {
	ClearFlag( RECEIVE_MODE );
	Set_TxRx_Labels();
	Close_Capture();
	first_call = TRUE;
	return( FALSE );
  }

  if( dec_chr < NO_CHARACTER )
  {
	/* Print character to Rx viewer */
	Print_Character( dec_chr, rx_text_buffer,
		rx_scrolledwindow, &iter );

	/* Reset char count on line feed */
	if( dec_chr == LF )
	  chr_cnt = 0;
	else
	{
	  /* Do word wrapping */
	  chr_cnt++;
	  if( (chr_cnt >= rc_data.word_wrap) &&
		  (dec_chr == ' ') )
	  {
		Print_Character( LF, rx_text_buffer,
			rx_scrolledwindow, &iter );
		chr_cnt = 0;
	  }
	} /* else */

  } /* if( dec_chr <= 127 ) */

  return( TRUE );
} /* Receive_Mode() */

/*------------------------------------------------------------------------*/

/*  Initialize_Transmit()
 *
 *  Initializes the Transmit mode
 */

  gboolean
Initialize_Transmit(gpointer data)
{
  int error;
  char mesg[MESG_SIZE];

  /* Setup sound card, abort on error */
  mesg[0] = '\0';
  if( !Open_Playback(mesg, &error) )
  {
	if( error )
	{
	  Strlcat( mesg, _("\nError: "), sizeof(mesg) );
	  Strlcat( mesg, snd_strerror(error), sizeof(mesg) );
	}
	Close_Playback();

	ClearFlag(
		TRANSMIT_KEYBD |
		TRANSMIT_MACRO |
		TRANSMIT_MODE );
	Set_TxRx_Labels();
	Error_Dialog( mesg, QUIT );
	return( FALSE );
  } /* if( !Setup_Sound_Card( mesg, &error ) ) */

  /* Set transceiver mode if CAT enabled, abort on error */
  SetFlag( TRANSMIT_MODE );
  if( isFlagSet(ENABLE_CAT) )
  {
	/* Switch to SSB and PTT ON */
	if( !Open_Tcvr_Serial() || !Set_Tcvr_Mode() )
	{
	  Close_Playback();
	  ClearFlag(
		  TRANSMIT_KEYBD |
		  TRANSMIT_MACRO |
		  TRANSMIT_MODE );
	  Set_TxRx_Labels();
	  return( FALSE );
	}
  }
  else if( isFlagSet(ENABLE_RTS_DTR) )
  {
	/* Open RTS control, abort on error */
	if( !Open_RTS_Serial() || !Set_Tcvr_Mode() )
	{
	  Close_Playback();
	  ClearFlag(
		  TRANSMIT_KEYBD |
		  TRANSMIT_MACRO |
		  TRANSMIT_MODE );
	  Set_TxRx_Labels();
	  return( FALSE );
	}
  }
  Set_TxRx_Labels();

  /* Transmit carrier and reversals, abort on error */
  if( strcmp(data, "from_macro") != 0 )
  {
	if( !Transmit_Preamble() )
	{
	  Close_Playback();
	  ClearFlag( TRANSMIT_MODE );
	  Set_TxRx_Labels();
	  return( FALSE );
	}
  }

  /* Open qso record file */
  if( !Open_Record_File() )
  {
	Close_Playback();
	ClearFlag(
		TRANSMIT_KEYBD |
		TRANSMIT_MACRO |
		TRANSMIT_MODE );
	Set_TxRx_Labels();
	return( FALSE );
  }

  return( TRUE );
} /* Initialize_Transmit() */

/*------------------------------------------------------------------------*/

